# Generated by Django 4.2.13 on 2024-06-04 15:16

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_easy_quiz", "0004_auto_20231009_1707"),
    ]

    operations = [
        migrations.AddField(
            model_name="savedquiz",
            name="interpretation_max_symbol",
            field=models.CharField(blank=True, max_length=32, null=True),
        ),
        migrations.AlterField(
            model_name="interpretationplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="interpretationplugin",
            name="title_level",
            field=models.CharField(
                choices=[
                    ("h1", "Titre niveau 1 (<h1>)"),
                    ("h2", "Titre niveau 2 (<h2>)"),
                    ("h3", "Titre niveau 3 (<h3>)"),
                    ("h4", "Titre niveau 4 (<h4>)"),
                    ("h5", "Titre niveau 5 (<h5>)"),
                    ("h6", "Titre niveau 6 (<h6>)"),
                ],
                help_text="Niveau du titre",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="moreinfoquestion",
            name="answer_is_list",
            field=models.BooleanField(
                default=False,
                help_text="Cochez cette case pour lister les réponses dans un élément 'select', plus compact que des boutons type 'radio'.",
            ),
        ),
        migrations.AlterField(
            model_name="weightedanswersplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                auto_created=True,
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="weightedanswersplugin",
            name="title_level",
            field=models.CharField(
                choices=[
                    ("h1", "Titre niveau 1 (<h1>)"),
                    ("h2", "Titre niveau 2 (<h2>)"),
                    ("h3", "Titre niveau 3 (<h3>)"),
                    ("h4", "Titre niveau 4 (<h4>)"),
                    ("h5", "Titre niveau 5 (<h5>)"),
                    ("h6", "Titre niveau 6 (<h6>)"),
                ],
                help_text="Niveau du titre",
                max_length=4,
            ),
        ),
    ]



# Django
from django.conf import settings
from django.utils.translation import gettext_lazy as _

GATHER_STATISTICS_DURING_QUIZ = getattr(
    settings, "DJANGO_EASY_QUIZ_GATHER_STATISTICS_DURING_QUIZ", False
)

SAVE_QUIZZES_RESULTS = getattr(settings, "DJANGO_EASY_QUIZ_SAVE_QUIZZES_RESULTS", False)

GATHER_STATISTICS = getattr(settings, "DJANGO_EASY_QUIZ_GATHER_STATISTICS", False)

SAVE_PDF = getattr(settings, "DJANGO_EASY_QUIZ_SAVE_PDF", False)

RELAUNCH_BUTTON = getattr(settings, "DJANGO_EASY_QUIZ_RELAUNCH_BUTTON", False)

PDF_FILE_NAME = getattr(
    settings, "DJANGO_EASY_QUIZ_PDF_FILE_NAME", _("quiz_summary.pdf")
)



# Django
from django.conf import settings
from django.urls import path
from django.utils.translation import gettext_lazy as _

# Local application / specific library imports
from .views.interpretation import InterpretationQuizView
from .views.misc import DisplayPdfView
from .views.weightedanswers import WeightedAnswersQuizView

urlpatterns = [
    path(
        _("weighted-answers/<int:pk>/"),
        WeightedAnswersQuizView.as_view(),
        name="weighted_answers_quiz",
    ),
    path(
        _("interpretation/<int:pk>/"),
        InterpretationQuizView.as_view(),
        name="interpretation_quiz",
    ),
]

if getattr(settings, "DJANGO_EASY_QUIZ_SAVE_PDF", False):
    urlpatterns += [
        path(
            "pdf/<uuid:identifier>", DisplayPdfView.as_view(), name="quiz_download_pdf"
        ),
    ]



# Django
from django.views.generic import FormView

# Project
from django_easy_quiz.forms import InterpretationQuizForm
from django_easy_quiz.models import (
    INTERPRETATION_QUESTION_CHOICES,
    InterpretationQuiz,
    MoreInfoAnswer,
    MoreInfoQuestion,
    SavedQuiz,
)
from django_easy_quiz.settings import (
    GATHER_STATISTICS,
    GATHER_STATISTICS_DURING_QUIZ,
    PDF_FILE_NAME,
    RELAUNCH_BUTTON,
    SAVE_PDF,
    SAVE_QUIZZES_RESULTS,
)
from django_easy_quiz.utils import get_more_info_quiz_formset, get_quiz_and_formset


class InterpretationQuizView(FormView):
    form_class = InterpretationQuizForm
    model = InterpretationQuiz
    template_name = "django_easy_quiz/interpretation_form.html"

    def get_success_url(self):
        return self.request.path

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        quiz, formset = get_quiz_and_formset(
            InterpretationQuiz,
            InterpretationQuizForm,
            self.kwargs["pk"],
        )

        if SAVE_QUIZZES_RESULTS:
            if (
                "saved_quiz" not in self.request.POST
                or not self.request.POST["saved_quiz"].isdigit()
            ):
                saved_quiz_id = SavedQuiz.objects.create(
                    quiz_type=str(quiz),
                    quiz_ended=False,
                ).id
                context["saved_quiz_id"] = saved_quiz_id

        if GATHER_STATISTICS:
            if quiz.more_info_quiz is not None:
                context["more_info_quiz"] = quiz.more_info_quiz

        context["quiz"] = quiz
        context["formset"] = formset
        context["pdf_file_name"] = PDF_FILE_NAME

        return context

    def handle_first_post_request(self, request, form, *args, **kwargs):
        if form.is_valid():
            context = self.get_context_data()

            answers = []  # noqa: FURB138
            for input in form.data:
                if "question_" in input:
                    # question_id = input.split("_")[1]
                    # answer_id = int(form.data[input])
                    # store position in INTERPRETATION_QUESTION_CHOICES:
                    answers.append(int(form.data[input]))
            quiz = InterpretationQuiz.objects.get(id=int(kwargs["pk"]))

            # return conclusion equal to max answers in INTERPRETATION_QUESTION_CHOICES (1=■, 2=▲, 3=◆, 4=●)
            # TODO: check for equality ?
            max_occurrences = max(answers, key=answers.count)
            context["symbol"] = INTERPRETATION_QUESTION_CHOICES[max_occurrences - 1][1]
            if max_occurrences == 1:
                context["conclusion"] = quiz.conclusion_max_1
            if max_occurrences == 2:
                context["conclusion"] = quiz.conclusion_max_2
            if max_occurrences == 3:
                context["conclusion"] = quiz.conclusion_max_3
            if max_occurrences == 4:
                context["conclusion"] = quiz.conclusion_max_4

            final_quiz = []
            questions = quiz.interpretationquizquestion_set.all()
            index = 0
            for question in questions:
                final_quiz.append(
                    {
                        "question": question.toJson(),
                        "answer": getattr(question, f"answer_{answers[index]}", ""),
                    }
                )
                index += 1

            # we might already have data for more_info_quiz here, so treat it here too
            more_info_quiz_answers = []
            for question in form.data:
                answer_object = None
                question_id = (
                    question.split("_")[2]
                    if question.startswith("more_question_")
                    else ""
                )
                if question_id.isdigit():
                    answer = form.data[question]
                    question = MoreInfoQuestion.objects.get(id=question_id).label
                    if answer.isdigit():
                        answer_object = MoreInfoAnswer.objects.get(id=answer).answer
                    else:
                        answer_object = answer
                    more_info_quiz_answers.append(
                        {"question": question, "answer": answer_object}
                    )

            if "saved_quiz" in request.POST and request.POST["saved_quiz"].isdigit():
                saved_quiz = SavedQuiz.objects.get(id=request.POST["saved_quiz"])
                saved_quiz.quiz_ended = True
                saved_quiz.interpretation_quiz = quiz
                saved_quiz.interpretation_max_symbol = context["symbol"]
                saved_quiz.answers = final_quiz
                saved_quiz.conclusion = context["conclusion"]
                saved_quiz.answers = final_quiz
                saved_quiz.more_infos = more_info_quiz_answers
                saved_quiz.save()

                context["saved_quiz_id"] = saved_quiz.id

                if GATHER_STATISTICS:
                    if quiz.more_info_quiz is not None:
                        formset = get_more_info_quiz_formset(quiz.more_info_quiz)
                        context["more_info_quiz"] = formset
                        context["more_info_quiz_after_main_quiz"] = True

            if GATHER_STATISTICS_DURING_QUIZ and "more_info_quiz" in context:
                context["more_info_quiz_after_main_quiz"] = False

            if SAVE_PDF and "saved_quiz_id" in context:
                context["weasyprint_download_uuid"] = saved_quiz.uuid

            context["relaunch_button"] = RELAUNCH_BUTTON

            return self.render_to_response(context)
        return self.form_invalid(form)

    def handle_second_post_request(self, request, form, *args, **kwargs):
        context = self.get_context_data()
        if "saved_quiz" in request.POST and request.POST["saved_quiz"].isdigit():
            saved_quiz = SavedQuiz.objects.get(id=request.POST["saved_quiz"])
            context["quiz"] = saved_quiz.interpretation_quiz
            context["final_quiz"] = saved_quiz.answers
            context["conclusion"] = saved_quiz.conclusion
            context["relaunch_button"] = RELAUNCH_BUTTON
            context["symbol"] = saved_quiz.interpretation_max_symbol
            context["saved_quiz_id"] = saved_quiz.id
            if SAVE_PDF and "saved_quiz_id" in context:
                context["weasyprint_download_uuid"] = saved_quiz.uuid

            answers = []
            for question in form.data:
                answer_object = None
                question_id = question.split("_")[2] if "question_" in question else ""
                if question_id.isdigit():
                    answer = form.data[question]
                    question = MoreInfoQuestion.objects.get(id=question_id).label
                    if answer.isdigit():
                        answer_object = MoreInfoAnswer.objects.get(id=answer).answer
                    else:
                        answer_object = answer
                    answers.append({"question": question, "answer": answer_object})

            saved_quiz.more_infos = answers
            saved_quiz.save()

        if SAVE_PDF and saved_quiz:
            context["weasyprint_download_uuid"] = saved_quiz.uuid

        context[
            "more_info_quiz"
        ] = False  # do not keep the form: do not try to display fields in template anymore
        context["thank_you"] = True  # thanks the person for the more_info quiz

        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        form = self.get_form()

        if not GATHER_STATISTICS_DURING_QUIZ:
            if "quiz_type" in form.data and form.data["quiz_type"] == "more_info":
                return self.handle_second_post_request(request, form, *args, **kwargs)
            return self.handle_first_post_request(request, form, *args, **kwargs)

        self.handle_first_post_request(request, form, *args, **kwargs)



# Generated by Django 3.2.24 on 2024-06-12 11:51

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0005_savedquiz_interpretation_max_symbol_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="weightedanswersquiz",
            name="show_score_summary",
            field=models.BooleanField(
                default=True, verbose_name="Show score in summary"
            ),
        ),
    ]



# Django
from django.views.generic import FormView

# Project
from django_easy_quiz.forms import WeightedAnswersQuizForm
from django_easy_quiz.models import (
    MoreInfoAnswer,
    MoreInfoQuestion,
    SavedQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizConclusion,
)
from django_easy_quiz.settings import (
    GATHER_STATISTICS,
    GATHER_STATISTICS_DURING_QUIZ,
    RELAUNCH_BUTTON,
    SAVE_PDF,
    SAVE_QUIZZES_RESULTS,
)
from django_easy_quiz.utils import get_more_info_quiz_formset, get_quiz_and_formset


class WeightedAnswersQuizView(FormView):
    form_class = WeightedAnswersQuizForm
    model = WeightedAnswersQuiz
    template_name = "django_easy_quiz/weighted_answers_form.html"

    def get_success_url(self):
        return self.request.path

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        quiz, formset = get_quiz_and_formset(
            WeightedAnswersQuiz,
            WeightedAnswersQuizForm,
            self.kwargs["pk"],
        )

        if SAVE_QUIZZES_RESULTS:
            if (
                "saved_quiz" not in self.request.POST
                or not self.request.POST["saved_quiz"].isdigit()
            ):
                saved_quiz_id = SavedQuiz.objects.create(
                    quiz_type=str(quiz),
                    quiz_ended=False,
                ).id
                context["saved_quiz_id"] = saved_quiz_id

        if GATHER_STATISTICS:
            if quiz.more_info_quiz is not None:
                more_info_quiz_formset = get_more_info_quiz_formset(quiz.more_info_quiz)
                context["more_info_quiz"] = more_info_quiz_formset
                context["more_info_quiz_after_main_quiz"] = True

        if GATHER_STATISTICS_DURING_QUIZ and "more_info_quiz" in context:
            context["more_info_quiz_after_main_quiz"] = False

        context["quiz"] = quiz
        context["formset"] = formset
        return context

    def handle_first_post_request(self, request, form, *args, **kwargs):
        if form.is_valid():
            context = self.get_context_data()

            quiz = WeightedAnswersQuiz.objects.get(id=int(kwargs["pk"]))
            total_points = 0
            final_quiz = []

            for input in form.data:
                answers = []
                if input.startswith("question_"):
                    question = quiz.weightedanswersquizquestion_set.get(
                        id=input.split("_")[1]
                    )
                    answers_ids = [int(id) for id in form.data.getlist(input)]
                    if 1 in answers_ids:
                        answers.append(question.answer_1)
                        total_points += question.points_answer_1
                    if 2 in answers_ids:
                        answers.append(question.answer_2)
                        total_points += question.points_answer_2
                    if 3 in answers_ids:
                        answers.append(question.answer_3)
                        total_points += question.points_answer_3
                    if 4 in answers_ids:
                        answers.append(question.answer_4)
                        total_points += question.points_answer_4
                    if 5 in answers_ids:
                        answers.append(question.answer_5)
                        total_points += question.points_answer_5
                    if 6 in answers_ids:
                        answers.append(question.answer_6)
                        total_points += question.points_answer_6
                    final_quiz.append({"question": question, "answers": answers})
            conclusion = WeightedAnswersQuizConclusion.objects.filter(
                min_points__lte=total_points, max_points__gte=total_points, quiz=quiz
            )
            if conclusion.exists():
                conclusion = conclusion[0]
            else:
                conclusion = ""

            conclusion.points = total_points
            context["conclusion"] = conclusion
            context["final_quiz"] = final_quiz
            context["relaunch_button"] = RELAUNCH_BUTTON

            # we might already have data for more_info_quiz here, so treat it here too
            more_info_quiz_answers = []
            for question in form.data:
                answer_object = None
                question_id = (
                    question.split("_")[2]
                    if question.startswith("more_question_")
                    else ""
                )
                if question_id.isdigit():
                    answer = form.data[question]
                    question = MoreInfoQuestion.objects.get(id=question_id)
                    if not question.answer_is_txt and answer.isdigit():
                        answer_object = MoreInfoAnswer.objects.get(id=answer).answer
                    else:
                        answer_object = answer
                    more_info_quiz_answers.append(
                        {"question": question.label, "answer": answer_object}
                    )

            if "saved_quiz" in request.POST and request.POST["saved_quiz"].isdigit():
                for question in final_quiz:
                    question["question"] = question["question"].toJson()
                saved_quiz = SavedQuiz.objects.get(id=request.POST["saved_quiz"])
                saved_quiz.quiz_ended = True
                saved_quiz.weighted_answers_quiz = quiz
                saved_quiz.conclusion = {
                    "points": conclusion.points,
                    "description": conclusion.description,
                }
                saved_quiz.answers = final_quiz
                saved_quiz.more_infos = more_info_quiz_answers
                saved_quiz.save()

                context["saved_quiz_id"] = saved_quiz.id

            if SAVE_PDF and "saved_quiz_id" in context:
                context["weasyprint_download_uuid"] = saved_quiz.uuid

            return self.render_to_response(context)
        return self.form_invalid(form)

    def handle_second_post_request(self, request, form, *args, **kwargs):
        context = self.get_context_data()
        if "saved_quiz" in request.POST and request.POST["saved_quiz"].isdigit():
            saved_quiz = SavedQuiz.objects.get(id=request.POST["saved_quiz"])

            context["quiz"] = saved_quiz.weighted_answers_quiz
            context["final_quiz"] = saved_quiz.answers
            context["conclusion"] = saved_quiz.conclusion
            context["relaunch_button"] = RELAUNCH_BUTTON

            answers = []
            for question in form.data:
                answer_object = None
                question_id = (
                    question.split("_")[2]
                    if question.startswith("more_question_")
                    else ""
                )
                if question_id.isdigit():
                    answer = form.data[question]
                    question = MoreInfoQuestion.objects.get(id=question_id)
                    if not question.answer_is_txt and answer.isdigit():
                        answer_object = MoreInfoAnswer.objects.get(id=answer).answer
                    else:
                        answer_object = answer
                    answers.append(
                        {"question": question.label, "answer": answer_object}
                    )

            saved_quiz.more_infos = answers
            saved_quiz.save()

        if SAVE_PDF and saved_quiz:
            context["weasyprint_download_uuid"] = saved_quiz.uuid

        context[
            "more_info_quiz"
        ] = False  # do not keep the form: do not try to display fields in template anymore
        context["thank_you"] = True  # thanks the person for the more_info quiz

        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        form = self.get_form()

        if not GATHER_STATISTICS_DURING_QUIZ:
            if "quiz_type" in form.data and form.data["quiz_type"] == "more_info":
                return self.handle_second_post_request(request, form, *args, **kwargs)
            return self.handle_first_post_request(request, form, *args, **kwargs)
        return self.handle_first_post_request(request, form, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        if "question" not in request.GET:
            return super().get(request, *args, **kwargs)

        context = self.get_context_data()
        quiz = WeightedAnswersQuiz.objects.get(id=int(kwargs["pk"]))
        questions_url = request.GET.getlist("question")
        quiz_context = []
        total_points = 0
        for question in questions_url:
            answers_objects = []
            answers = [int(id) for id in question.split(".")[1:]]
            question = quiz.weightedanswersquizquestion_set.get(
                id=int(question.split(".")[0])
            )
            if 1 in answers:
                total_points += question.points_answer_1
                answers_objects.append(question.answer_1)
            if 2 in answers:
                total_points += question.points_answer_2
                answers_objects.append(question.answer_2)
            if 3 in answers:
                total_points += question.points_answer_3
                answers_objects.append(question.answer_3)
            if 4 in answers:
                total_points += question.points_answer_4
                answers_objects.append(question.answer_4)
            if 5 in answers:
                total_points += question.points_answer_5
                answers_objects.append(question.answer_5)
            if 6 in answers:
                total_points += question.points_answer_6
                answers_objects.append(question.answer_6)

            quiz_context.append({"question": question, "answers": answers_objects})

        quiz.points = total_points
        quiz.conclusion_description = WeightedAnswersQuizConclusion.objects.filter(
            min_points__lte=total_points, max_points__gte=total_points, quiz=quiz
        )[0].description
        context["quiz"] = quiz
        context["quiz_answers"] = quiz_context

        return self.render_to_response(context)



# Django
from django.utils.translation import gettext as _

try:
    # Third party
    from cms.plugin_base import CMSPluginBase
    from cms.plugin_pool import plugin_pool

    # Local application / specific library imports
    from .models import InterpretationPlugin, WeightedAnswersPlugin

    @plugin_pool.register_plugin
    class WeightedAnswersPluginPublisher(CMSPluginBase):
        module = _("Quiz")
        name = _("Weighted Answers Quiz")
        model = WeightedAnswersPlugin
        render_template = "django_easy_quiz/weighted_answers_plugin.html"

        allow_children = False

        def render(self, context, instance, placeholder):
            context.update({"instance": instance})
            return context

    @plugin_pool.register_plugin
    class InterpretationPluginPublisher(CMSPluginBase):
        module = _("Quiz")
        name = _("Interpretation Quiz")
        model = InterpretationPlugin
        render_template = "django_easy_quiz/interpretation_plugin.html"
        allow_children = False

        def render(self, context, instance, placeholder):
            context.update({"instance": instance})
            return context

except ImportError:
    ...



# Standard Library
from io import BytesIO

# Django
from django.http import FileResponse
from django.template.loader import render_to_string
from django.utils.datastructures import MultiValueDictKeyError
from django.views import generic

# Project
from django_easy_quiz.models import SavedQuiz


class DisplayPdfView(generic.detail.BaseDetailView):
    def get(self, request, identifier=None, *args, **kwargs):
        # Third party
        from weasyprint import HTML

        try:
            fname = request.GET["name"] + ".pdf"
        except MultiValueDictKeyError:
            fname = "document.pdf"

        saved_quiz = SavedQuiz.objects.filter(uuid=identifier).first()

        quiz_type = (
            "interpretation_quiz"
            if saved_quiz.interpretation_quiz
            else "weighed_answers_quiz"
        )
        final_quiz = saved_quiz.answers
        conclusion = saved_quiz.conclusion

        rendered = render_to_string(
            "django_easy_quiz/quiz_pdf.html",
            {
                "quiz_type": quiz_type,
                "final_quiz": final_quiz,
                "conclusion": conclusion,
            },
        )
        html = HTML(string=rendered, base_url=request.build_absolute_uri())
        content = BytesIO(html.write_pdf())
        response = FileResponse(content, content_type="application/pdf")
        response["Content-Disposition"] = "filename={}".format(fname)
        response["Content-Length"] = content.getbuffer().nbytes
        return response
